Beaver.Env (beaver v0.4.8)

This module defines macros to getting MLIR context, region, block within the do block of mlir/1. It works like MODULE/0, CALLER/0 of Elixir special forms.

Summary

Functions

Return block in the DSL environment

Return block with a given name in the DSL environment

Return context in the DSL environment

Return insertion point in the DSL environment

Return region in the DSL environment

Functions

block()

(macro)

Return block in the DSL environment

block(block_var)

(macro)

Return block with a given name in the DSL environment

context()

(macro)

Return context in the DSL environment

ip()

(macro)

Return insertion point in the DSL environment

region()

(macro)

Return region in the DSL environment

valid_insertion_point?(ip)