Beaver.MLIR.ConversionPattern (beaver v0.4.8)

Callback-backed dialect conversion patterns.

add/5 creates a pattern and immediately transfers its ownership to the rewrite pattern set, so a standalone native pattern cannot be leaked.

Summary

Types

callback()

@type callback() :: (Beaver.MLIR.Operation.t(),
               [Beaver.MLIR.Value.t()]
               | [[Beaver.MLIR.Value.t()]],
               Beaver.MLIR.ConversionPatternRewriter.t() ->
                 :ok | :no_match | {:error, term()})

t()

@type t() :: %Beaver.MLIR.ConversionPattern{ref: term()}

Functions

add(set, root_name, type_converter, callback, opts \\ [])

make(value)