Beaver. MLIR. Dialect. Builtin
(beaver v0.4.8)
This module defines functions for Ops in Builtin dialect.
Summary
Functions
Macro to create a module and insert ops into its body. Beaver.region/1 shouldn't be called because module's one-block region will be created.
Return op name builtin.unrealized_conversion_cast as a bitstring.
builtin.unrealized_conversion_cast
Functions
Macro to create a module and insert ops into its body. Beaver.region/1 shouldn't be called because module's one-block region will be created.
Return op name builtin.unrealized_conversion_cast as a bitstring.
builtin.unrealized_conversion_cast