Beaver.MLIR.Dialect.PDL (beaver v0.4.7)

Summary

Functions

pdl.apply_native_constraint - Apply a native constraint to a set of provided entities

pdl.apply_native_rewrite - Apply a native rewrite method inside of pdl.rewrite region

pdl.attribute - Define an input attribute in a pattern

pdl.erase - Mark an input operation as erased

pdl.operand - Define an external input operand in a pattern

pdl.operands - Define a range of input operands in a pattern

pdl.operation - Define an operation within a pattern

pdl.pattern

pdl.range - Construct a range of pdl entities

pdl.replace - Mark an input operation as replaced

pdl.result - Extract a result from an operation

pdl.results - Extract a result group from an operation

pdl.rewrite - Specify the rewrite of a matched pattern

pdl.type

pdl.types

Functions

apply_native_constraint(ssa)

pdl.apply_native_constraint - Apply a native constraint to a set of provided entities

Attributes

  • name - Single, StrAttr, string attribute
  • isNegated - Single, BoolAttr, bool attribute

Operands

  • args - Variadic, PDL_AnyType, variadic of pdl type

Results

  • results - Variadic, PDL_AnyType, variadic of pdl type

Description

pdl.apply_native_constraint operations apply a native C++ constraint, that has been registered externally with the consumer of PDL, to a given set of entities and optionally return a number of values.

Example:

// Apply `myConstraint` to the entities defined by `input`, `attr`, and `op`.
pdl.apply_native_constraint "myConstraint"(%input, %attr, %op : !pdl.value, !pdl.attribute, !pdl.operation)
// Apply constraint `with_result` to `root`. This constraint returns an attribute.
%attr = pdl.apply_native_constraint "with_result"(%root : !pdl.operation) : !pdl.attribute

apply_native_rewrite(ssa)

pdl.apply_native_rewrite - Apply a native rewrite method inside of pdl.rewrite region

Attributes

  • name - Single, StrAttr, string attribute

Operands

  • args - Variadic, PDL_AnyType, variadic of pdl type

Results

  • results - Variadic, PDL_AnyType, variadic of pdl type

Description

pdl.apply_native_rewrite operations apply a native C++ function, that has been registered externally with the consumer of PDL, to perform a rewrite and optionally return a number of values. The native function may accept any number of arguments. This operation is used within a pdl.rewrite region to enable the interleaving of native rewrite methods with other pdl constructs.

Example:

// Apply a native rewrite method that returns an attribute.
%ret = pdl.apply_native_rewrite "myNativeFunc"(%arg0, %attr1) : !pdl.attribute
// The native rewrite as defined in C++:
static Attribute myNativeFunc(PatternRewriter &rewriter, Value arg0, Attribute arg1) {
  // Just return the second arg.
  return arg1;
}

void registerNativeRewrite(PDLPatternModule &pdlModule) {
  pdlModule.registerRewriteFunction("myNativeFunc", myNativeFunc);
}

attribute(ssa)

pdl.attribute - Define an input attribute in a pattern

Attributes

  • value - Optional, AnyAttr, any attribute

Operands

  • valueType - Optional, PDL_Type, PDL handle to an mlir::Type

Results

  • attr - Single, PDL_Attribute, PDL handle to an mlir::Attribute

Description

pdl.attribute operations capture named attribute edges into an operation. Instances of this operation define, and partially constrain, attributes of a given operation. A pdl.attribute may partially constrain the input by specifying an expected attribute value type (via a pdl.type operation), or a constant value for the attribute (via val). Only one of these may be set for a given input, as the type of the constant value provides the type. When defined within a pdl.rewrite region, the constant value must be specified.

Example:

// Define an attribute:
%attr = pdl.attribute

// Define an attribute with an expected type:
%type = pdl.type : i32
%attr = pdl.attribute : %type

// Define an attribute with a constant value:
%attr = pdl.attribute = "hello"

erase(ssa)

pdl.erase - Mark an input operation as erased

Operands

  • opValue - Single, PDL_Operation, PDL handle to an mlir::Operation *

Description

pdl.erase operations are used within pdl.rewrite regions to specify that an input operation should be marked as erased. The semantics of this operation correspond with the eraseOp method on a PatternRewriter.

Example:

pdl.erase %root

operand(ssa)

pdl.operand - Define an external input operand in a pattern

Operands

  • valueType - Optional, PDL_Type, PDL handle to an mlir::Type

Results

  • value - Single, PDL_Value, PDL handle for an mlir::Value

Description

pdl.operand operations capture external operand edges into an operation node that originate from operations or block arguments not otherwise specified within the pattern (i.e. via pdl.result or pdl.results). These operations define individual operands of a given operation. A pdl.operand may partially constrain an operand by specifying an expected value type (via a pdl.type operation).

Example:

// Define an external operand:
%operand = pdl.operand

// Define an external operand with an expected type:
%type = pdl.type : i32
%operand = pdl.operand : %type

operands(ssa)

pdl.operands - Define a range of input operands in a pattern

Operands

  • valueType - Optional, anonymous/composite constraint, range of PDL handle to an mlir::Type values

Results

  • value - Single, anonymous/composite constraint, range of PDL handle for an mlir::Value values

Description

pdl.operands operations capture external operand range edges into an operation node that originate from operations or block arguments not otherwise specified within the pattern (i.e. via pdl.result or pdl.results). These operations define groups of input operands into a given operation. A pdl.operands may partially constrain a set of input operands by specifying expected value types (via pdl.types operations).

Example:

// Define a range of input operands:
%operands = pdl.operands

// Define a range of input operands with expected types:
%types = pdl.types : [i32, i64, i32]
%typed_operands = pdl.operands : %types

operation(ssa)

pdl.operation - Define an operation within a pattern

Attributes

  • opName - Optional, StrAttr, string attribute
  • attributeValueNames - Single, StrArrayAttr, string array attribute

Operands

  • operandValues - Variadic, anonymous/composite constraint, variadic of single element or range of PDL handle for an mlir::Value
  • attributeValues - Variadic, PDL_Attribute, variadic of PDL handle to an mlir::Attribute
  • typeValues - Variadic, anonymous/composite constraint, variadic of single element or range of PDL handle to an mlir::Type

Results

  • op - Single, PDL_Operation, PDL handle to an mlir::Operation *

Description

pdl.operation operations define operation nodes within a pattern. Within a match sequence, i.e. when directly nested within a pdl.pattern, these operations correspond to input operations, or those that already existing within the MLIR module. Inside of a pdl.rewrite, these operations correspond to operations that should be created as part of the replacement sequence.

pdl.operations are composed of a name, and a set of attribute, operand, and result type values, that map to what those that would be on a constructed instance of that operation. The results of a pdl.operation are a handle to the operation itself. Handles to the results of the operation can be extracted via pdl.result.

Example:

// Define an instance of a `foo.op` operation.
%op = pdl.operation "foo.op"(%arg0, %arg1 : !pdl.value, !pdl.value)
  {"attrA" = %attr0} -> (%type, %type : !pdl.type, !pdl.type)

When used within a matching context, the name of the operation may be omitted.

When used within a rewriting context, i.e. when defined within a pdl.rewrite, all of the result types must be "inferable". This means that the type must be attributable to either a constant type value or the result type of another entity, such as an attribute, the result of a apply_native_rewrite, or the result type of another operation. If the result type value does not meet any of these criteria, the operation must override the InferTypeOpInterface to ensure that the result types can be inferred.

The operands of the operation are interpreted in the following ways:

  1. A single !pdl.range<value>:

In this case, the single range is treated as all of the operands of the operation.

// Define an instance with single range of operands.
%op = pdl.operation "func.return"(%allArgs : !pdl.range<value>)
  1. A variadic number of either !pdl.value or !pdl.range<value>:

In this case, the inputs are expected to correspond with the operand groups defined on the operation in ODS.

// Given the following operation definition in ODS:
def MyIndirectCallOp {
  let results = (outs FunctionType:$call, Variadic<AnyType>:$args);
}
// We can match the operands as so:
%op = pdl.operation "my.indirect_call"(%call, %args : !pdl.value, !pdl.range<value>)

The results of the operation are interpreted in the following ways:

  1. A single !pdl.range<type>:

In this case, the single range is treated as all of the result types of the operation.

// Define an instance with single range of types.
%allResultTypes = pdl.types
%op = pdl.operation "builtin.unrealized_conversion_cast" -> (%allResultTypes : !pdl.types)
  1. A variadic number of either !pdl.type or !pdl.range<type>:

In this case, the inputs are expected to correspond with the result groups defined on the operation in ODS.

// Given the following operation definition in ODS:
def MyOp {
  let results = (outs SomeType:$result, Variadic<SomeType>:$otherResults);
}
// We can match the results as so:
%result = pdl.type
%otherResults = pdl.types
%op = pdl.operation "foo.op" -> (%result, %otherResults : !pdl.type, !pdl.range<type>)

pattern(ssa)

pdl.pattern

range(ssa)

pdl.range - Construct a range of pdl entities

Operands

  • arguments - Variadic, PDL_AnyType, variadic of pdl type

Results

  • result - Single, anonymous/composite constraint, range of PDL handle to an mlir::Type or PDL handle for an mlir::Value values

Description

pdl.range operations construct a range from a given set of PDL entities, which all share the same underlying element type. For example, a !pdl.range<value> may be constructed from a list of !pdl.value or !pdl.range<value> entities.

Example:

// Construct a range of values.
%valueRange = pdl.range %inputValue, %inputRange : !pdl.value, !pdl.range<value>

// Construct a range of types.
%typeRange = pdl.range %inputType, %inputRange : !pdl.type, !pdl.range<type>

// Construct an empty range of types.
%valueRange = pdl.range : !pdl.range<type>

TODO: Range construction is currently limited to rewrites, but it could be extended to constraints under certain circustances; i.e., if we can determine how to extract the underlying elements. If we can't, e.g. if there are multiple sub ranges used for construction, we won't be able to determine their sizes during constraint time.

replace(ssa)

pdl.replace - Mark an input operation as replaced

Operands

  • opValue - Single, PDL_Operation, PDL handle to an mlir::Operation *
  • replOperation - Optional, PDL_Operation, PDL handle to an mlir::Operation *
  • replValues - Variadic, anonymous/composite constraint, variadic of single element or range of PDL handle for an mlir::Value

Description

pdl.replace operations are used within pdl.rewrite regions to specify that an input operation should be marked as replaced. The semantics of this operation correspond with the replaceOp method on a PatternRewriter. The set of replacement values can be either:

  • a single Operation (replOperation should be populated)
    • The operation will be replaced with the results of this operation.
  • a set of Values (replValues should be populated)
    • The operation will be replaced with these values.

Example:

// Replace root node with 2 values:
pdl.replace %root with (%val0, %val1 : !pdl.value, !pdl.value)

// Replace root node with a range of values:
pdl.replace %root with (%vals : !pdl.range<value>)

// Replace root with another operation:
pdl.replace %root with %otherOp

result(ssa)

pdl.result - Extract a result from an operation

Attributes

  • index - Single, I32Attr, 32-bit signless integer attribute

Operands

  • parent - Single, PDL_Operation, PDL handle to an mlir::Operation *

Results

  • val - Single, PDL_Value, PDL handle for an mlir::Value

Description

pdl.result operations extract result edges from an operation node within a pattern or rewrite region. The provided index is zero-based, and represents the concrete result to extract, i.e. this is not the result index as defined by the ODS definition of the operation.

Example:

// Extract a result:
%operation = pdl.operation ...
%pdl_result = pdl.result 1 of %operation

// Imagine the following IR being matched:
%result_0, %result_1 = foo.op ...

// If the example pattern snippet above were matching against `foo.op` in
// the IR snippet, `%pdl_result` would correspond to `%result_1`.

results(ssa)

pdl.results - Extract a result group from an operation

Attributes

  • index - Optional, I32Attr, 32-bit signless integer attribute

Operands

  • parent - Single, PDL_Operation, PDL handle to an mlir::Operation *

Results

  • val - Single, anonymous/composite constraint, integer or index

Description

pdl.results operations extract a result group from an operation within a pattern or rewrite region. If an index is provided, this operation extracts a result group as defined by the ODS definition of the operation. In this case the result of this operation may be either a single pdl.value or a pdl.range<value>, depending on the constraint of the result in ODS. If no index is provided, this operation extracts the full result range of the operation.

Example:

// Extract all of the results of an operation:
%operation = pdl.operation ...
%results = pdl.results of %operation

// Extract the results in the first result group of an operation, which is
// variadic:
%operation = pdl.operation ...
%results = pdl.results 0 of %operation -> !pdl.range<value>

// Extract the results in the second result group of an operation, which is
// not variadic:
%operation = pdl.operation ...
%results = pdl.results 1 of %operation -> !pdl.value

rewrite(ssa)

pdl.rewrite - Specify the rewrite of a matched pattern

Attributes

  • name - Optional, StrAttr, string attribute

Operands

  • root - Optional, PDL_Operation, PDL handle to an mlir::Operation *
  • externalArgs - Variadic, PDL_AnyType, variadic of pdl type

Description

pdl.rewrite operations terminate the region of a pdl.pattern and specify the main rewrite of a pdl.pattern, on the optional root operation. The rewrite is specified either via a string name (name) to a native rewrite function, or via the region body. The rewrite region, if specified, must contain a single block. If the rewrite is external it functions similarly to pdl.apply_native_rewrite, and takes a set of additional positional values defined within the matcher as arguments. If the rewrite is external, the root operation is passed to the native function as the leading arguments. The root operation, if provided, specifies the starting point in the pattern for the subgraph isomorphism search. Pattern matching will proceed from this node downward (towards the defining operation) or upward (towards the users) until all the operations in the pattern have been matched. If the root is omitted, the pdl_interp lowering will automatically select the best root of the pdl.rewrite among all the operations in the pattern.

Example:

// Specify an external rewrite function:
pdl.rewrite %root with "myExternalRewriter"(%value : !pdl.value)

// Specify a rewrite inline using PDL with the given root:
pdl.rewrite %root {
  %op = pdl.operation "foo.op"(%arg0, %arg1)
  pdl.replace %root with %op
}

// Specify a rewrite inline using PDL, automatically selecting root:
pdl.rewrite {
  %op1 = pdl.operation "foo.op"(%arg0, %arg1)
  %op2 = pdl.operation "bar.op"(%arg0, %arg1)
  pdl.replace %root1 with %op1
  pdl.replace %root2 with %op2
}

type(ssa)

pdl.type

types(ssa)

pdl.types