Beaver.MLIR.OpOperand (beaver v0.4.8)

This module defines functions working with MLIR OpOperand.

Summary

Functions

Returns the operand position within the owner operation.

Returns the operation that owns this operand use.

Returns the value currently referenced by this operand use.

Types

t()

@type t() :: %Beaver.MLIR.OpOperand{ref: term()}

Functions

make(value)

operand_number(op_operand)

@spec operand_number(t()) :: non_neg_integer()

Returns the operand position within the owner operation.

owner(op_operand)

@spec owner(t()) :: Beaver.MLIR.Operation.t()

Returns the operation that owns this operand use.

value(op_operand)

@spec value(t()) :: Beaver.MLIR.Value.t()

Returns the value currently referenced by this operand use.