Beaver. Native. OpaquePtr
(beaver v0.4.8)
This module defines functions working with opaque pointer, usually a void* in C.
Summary
Functions
read the N bytes starting from the pointer and returns an Erlang binary
read the N bytes starting from the pointer and return a resource
Types
@type t() :: %Beaver.Native.OpaquePtr{ref: term()}