Beaver. MLIR. Context
(beaver v0.4.8)
This module defines functions creating or destroying MLIR context.
Summary
Functions
Create a MLIR context. By default it registers and loads all dialects.
Check if the op name is terminator
Run a function with a registry appended to the context.
Types
@type context_option() :: {:allow_unregistered, boolean()} | {:all_dialects, boolean()} | {:threading, boolean()} | {:thread_pool, :application | Beaver.MLIR.LLVMThreadPool.t() | GenServer.server() | nil} | {:registry, Beaver.MLIR.DialectRegistry.t() | [Beaver.MLIR.DialectRegistry.t()]}
Functions
@spec create([context_option()]) :: t()
Create a MLIR context. By default it registers and loads all dialects.
Check if the op name is terminator
Run a function with a registry appended to the context.