Beaver. MLIR. Dialect behaviour
(beaver v0.4.8)
This module defines macro to generate code for an MLIR dialect.
You might override eval_ssa/1 function to introduce your custom op generation
Summary
Functions
Return the dialect name acc as a bitstring.
Return the dialect name affine as a bitstring.
Return the dialect name amdgpu as a bitstring.
Return the dialect name arith as a bitstring.
Return the dialect name arm_neon as a bitstring.
Return the dialect name arm_sme as a bitstring.
Return the dialect name arm_sve as a bitstring.
Return the dialect name async as a bitstring.
Return the dialect name bufferization as a bitstring.
Return the dialect name builtin as a bitstring.
Return the dialect name cf as a bitstring.
Return the dialect name complex as a bitstring.
Return the dialect name dlti as a bitstring.
Return the dialect name emitc as a bitstring.
Return the dialect name func as a bitstring.
Return the dialect name gpu as a bitstring.
Return the dialect handle for the given dialect name.
Return the dialect name index as a bitstring.
Return the dialect name irdl as a bitstring.
Return the dialect name linalg as a bitstring.
Return the dialect name llvm as a bitstring.
Return the dialect name math as a bitstring.
Return the dialect name memref as a bitstring.
Return the dialect name ml_program as a bitstring.
Return the dialect name mpi as a bitstring.
Return the dialect name nvgpu as a bitstring.
Return the dialect name nvvm as a bitstring.
Return the dialect name omp as a bitstring.
Return the dialect name pdl as a bitstring.
Return the dialect name pdl_interp as a bitstring.
Return the dialect name ptr as a bitstring.
Return the dialect name quant as a bitstring.
Return the dialect name rocdl as a bitstring.
Return the dialect name scf as a bitstring.
Return the dialect name shape as a bitstring.
Return the dialect name shard as a bitstring.
Return the dialect name smt as a bitstring.
Return the dialect name sparse_tensor as a bitstring.
Return the dialect name spirv as a bitstring.
Return the dialect name tensor as a bitstring.
Return the dialect name tosa as a bitstring.
Return the dialect name transform as a bitstring.
Return the dialect name ub as a bitstring.
Return the dialect name vector as a bitstring.
Return the dialect name wasmssa as a bitstring.
Return the dialect name x86 as a bitstring.
Return the dialect name xegpu as a bitstring.
Return the dialect name xevm as a bitstring.
Types
@type t() :: %Beaver.MLIR.Dialect{ref: term()}
Callbacks
@callback eval_ssa(Beaver.SSA.t()) :: any()
Functions
Return the dialect name acc as a bitstring.
Return the dialect name affine as a bitstring.
Return the dialect name amdgpu as a bitstring.
Return the dialect name arith as a bitstring.
Return the dialect name arm_neon as a bitstring.
Return the dialect name arm_sme as a bitstring.
Return the dialect name arm_sve as a bitstring.
Return the dialect name async as a bitstring.
Return the dialect name bufferization as a bitstring.
Return the dialect name builtin as a bitstring.
Return the dialect name cf as a bitstring.
Return the dialect name complex as a bitstring.
Return the dialect name dlti as a bitstring.
Return the dialect name emitc as a bitstring.
Return the dialect name func as a bitstring.
Return the dialect name gpu as a bitstring.
Return the dialect handle for the given dialect name.
Note that no every dialect has a handle function defined in the C API.
In that case, nil is returned.
Return the dialect name index as a bitstring.
Return the dialect name irdl as a bitstring.
Return the dialect name linalg as a bitstring.
Return the dialect name llvm as a bitstring.
Return the dialect name math as a bitstring.
Return the dialect name memref as a bitstring.
Return the dialect name ml_program as a bitstring.
Return the dialect name mpi as a bitstring.
Return the dialect name nvgpu as a bitstring.
Return the dialect name nvvm as a bitstring.
Return the dialect name omp as a bitstring.
Return the dialect name pdl as a bitstring.
Return the dialect name pdl_interp as a bitstring.
Return the dialect name ptr as a bitstring.
Return the dialect name quant as a bitstring.
Return the dialect name rocdl as a bitstring.
Return the dialect name scf as a bitstring.
Return the dialect name shape as a bitstring.
Return the dialect name shard as a bitstring.
Return the dialect name smt as a bitstring.
Return the dialect name sparse_tensor as a bitstring.
Return the dialect name spirv as a bitstring.
Return the dialect name tensor as a bitstring.
Return the dialect name tosa as a bitstring.
Return the dialect name transform as a bitstring.
Return the dialect name ub as a bitstring.
Return the dialect name vector as a bitstring.
Return the dialect name wasmssa as a bitstring.
Return the dialect name x86 as a bitstring.
Return the dialect name xegpu as a bitstring.
Return the dialect name xevm as a bitstring.